<?php

/**
 * @file
 * Cart page callbacks for the freecommerce module.
 */

/**
 * Form constructor for the freecommerce cart.
 *
 * @see freecommerce_cart_submit()
 *
 * @ingroup forms
 */
function freecommerce_cart($form, &$form_state) {
  if (isset($_SESSION['freecommerce_cart']) && count($_SESSION['freecommerce_cart']) >= 1) {
    require 'freecommerce_fields.inc.php';
    $fields = variable_get('freecommerce_address_fields', array('email'));
    // create containers for product (i.e., node title), quantity, and remove
    // so we can have nested values for the _submit handler
    $form['remove'] = array(
      '#tree' => TRUE,
    );
    $form['quantity'] = array(
      '#tree' => TRUE,
    );
    $form['product'] = array(
      '#tree' => TRUE,
    );

    $ok_nodes = variable_get('freecommerce_nodes', array());
    foreach ($_SESSION['freecommerce_cart'] as $id => $product) {
      $node = node_load($id);
      if (node_access('view', $node) && in_array($node->type, $ok_nodes)) { // double-check that node is still freeCommercized
        $form['product'][$node->nid] = array(
          '#markup' => check_plain($product['title']),
        );
        $form['quantity'][$node->nid] = array(
          '#type' => 'textfield',
          '#size' => 1,
          '#maxlength' => 3,
          '#default_value' => $product['quantity'],
        );
        $form['remove'][$node->nid] = array(
          '#type' => 'checkbox',
          '#return_value' => 1,
          '#default_value' => 0,
        );
      }
      else { // if not a legit node, remove it from session and, thus, the cart
        unset($_SESSION['freecommerce_cart'][$id]);
      }
    }
  }

  $form['checkout'] = array(
    '#type' => 'submit',
    '#value' => t('Checkout'),
  );

  $form['update'] = array(
    '#type' => 'submit',
    '#value' => t('Update order'),
  );

  $form['#submit'][] = 'freecommerce_cart_submit';

  return $form;
}

/**
 * Form submission handler for freecommerce_cart().
 *
 * @see user_login_form_validate()
 */
function freecommerce_cart_submit($form, &$form_state) {
  // process quantity / remove requests
  // regardless of which submit button has been pressed
  // this SEEMS like decent UX
  foreach ($form_state['values']['quantity'] as $key => $value) {
    if ($value) {
      $node = node_load($key);
      freecommerce_add_to_cart($node, $value);
    }
    if ($value === 0) { // if quantity set 0, simply remove
      unset($_SESSION['freecommerce_cart'][$key]);
    }
  }
  foreach ($form_state['values']['remove'] as $key => $value) {
    if ($value && $_SESSION['freecommerce_cart'][$key]) {
      unset($_SESSION['freecommerce_cart'][$key]);
    }
  }

  if ($form_state['values']['op'] == t('Update order')) {
    $form_state['redirect'] = 'cart';
  }
  else {
    $form_state['redirect'] = 'checkout';
  }
}

/**
 * Page callback: Places a node/item into cart.
 *
 * Places a node/item into cart (if possible), then redirects to /cart
 *
 * @param object $node
 *   Node being added to cart.
 *
 * @see freecommerce_menu()
 */
function freecommerce_cart_add($node) {
  $ok_nodes = variable_get('freecommerce_nodes', array());
  if (node_access('view', $node) && in_array($node->type, $ok_nodes)) {
    // product added to cart (or one more of existing quantity)
    freecommerce_add_to_cart($node);
  }
  // goto cart page -- without $nid argument
  drupal_goto('cart');
}

/**
 * Places node into cart by adjust $_SESSION['freecommerce_cart']
 *
 * @param object $node
 *   Node being added to cart.
 *
 * @param int $value
 *   Number of items (of type $node->title) being added to cart
 */
function freecommerce_add_to_cart($node, $value = 0) {
  if (!isset($_SESSION['freecommerce_cart'][$node->nid])) {
    $_SESSION['freecommerce_cart'][$node->nid] = array(
      'title' => $node->title,
      'quantity' => 0,
    );
  }
  $max = variable_get('freecommerce_max_order', 10);
  $name = check_plain($node->title);
  if ($value == 0) { // if no value is set, we just increment current value by 1
    $value = $_SESSION['freecommerce_cart'][$node->nid]['quantity'] + 1; 
  }
  if ($value > $max) {  // we've maxed out
    $_SESSION['freecommerce_cart'][$node->nid]['quantity'] = $max;
    drupal_set_message(t('You can only order a maximum of') . ' ' . $max . ' ' . $name . '.');
  }
  else {
    $_SESSION['freecommerce_cart'][$node->nid]['quantity'] = $value;
  }
}

/**
 * Turns $_SESSION['freecommerce_cart'] data into form/table cart.
 */
function freecommerce_product_table() {
  $ok_nodes = variable_get('freecommerce_nodes', array());
  foreach ($_SESSION['freecommerce_cart'] as $id => $product) {
    $node = node_load($id);
    if (node_access('view', $node) && in_array($node->type, $ok_nodes)) { // double-check that node is still free-commercized
      $row = array(
        check_plain($product['title']),
        $product['quantity'],
        t('Free'),
      );
      $rows[] = $row;
    }
  }
  if (empty($rows)) {
    return '';
  }

  $header = array(
    array(
      'data' => t('Product'),
    ),
    array(
      'data' => t('Quantity'),
    ),
    array(
      'data' => t('Price'),
    ),
  );
  $attrs = array('class' => 'freecommerce-table');
  return theme('table', $header, $rows, $attrs);
}